@@ -6,6 +6,7 @@
    :toctree: _autosummary
 
    ccobra.benchmark
+   ccobra.encoders
    ccobra.propositional
    ccobra.syllogistic
    ccobra.syllogistic_generalized
@@ -14,8 +15,6 @@
 
 .. autoclass:: CCobraData
     :members:
-.. autoclass:: CCobraDomainEncoder
-    :members:
 .. autoclass:: CCobraModel
     :members:
 .. autoclass:: Item
@@ -24,15 +23,15 @@
 """
 
 
+from .version import __version__
+
 from .data import CCobraData
 from .item import Item
 from .model import CCobraModel
-from .domainhandler import CCobraDomainEncoder
 
+from . import encoders
 from . import benchmark
-
 from . import propositional
 from . import syllogistic
 from . import syllogistic_generalized
 
-__version__ = '1.1.0'
