@@ -1,5 +1,12 @@
 """ CCOBRA benchmark functionality.
 
+.. rubric:: Submodules
+
+.. autosummary::
+   :toctree: _autosummary
+
+   ccobra.benchmark.comparator
+
 .. rubric:: Functions
 
 .. autofunction:: dir_context
@@ -9,7 +16,6 @@
 .. autofunction:: main
 .. autofunction:: parse_arguments
 .. autofunction:: silence_stdout
-.. autofunction:: tuple_to_string
 
 .. rubric:: Classes
 
@@ -17,12 +23,6 @@
     :members:
 .. autoclass:: ModelInfo
     :members:
-.. autoclass:: Comparator
-    :members:
-.. autoclass:: EqualityComparator
-    :members:
-.. autoclass:: NVCComparator
-    :members:
 .. autoclass:: Evaluator
     :members:
 .. autoclass:: ModelImporter
@@ -30,9 +30,11 @@
 
 """
 
+from . import comparator
+
 from .benchmark import Benchmark, ModelInfo, fix_rel_path, fix_model_path
-from .comparator import tuple_to_string, Comparator, EqualityComparator, NVCComparator
 from .contextmanager import dir_context
 from .evaluator import Evaluator
 from .modelimporter import ModelImporter
 from .runner import entry_point, parse_arguments, main, silence_stdout
+from .evaluation_handler import EvaluationHandler
