@@ -11,10 +11,11 @@ import pandas as pd
 from . import comparator
 from . import contextmanager
 from . import modelimporter
+from . import evaluation_handler
+from .. import encoders
 from ..data import CCobraData
-from ..domainhandler import CCobraDomainEncoder
-from ..propositional.encoder_prop import PropositionalEncoder
-from ..syllogistic.encoder_syl import SyllogisticEncoder
+from ..propositional.task_encoder_prop import PropositionalTaskEncoder
+from ..syllogistic.task_encoder_syl import SyllogisticTaskEncoder
 
 # Initialize logging
 logger = logging.getLogger(__name__)
@@ -96,40 +97,102 @@ def fix_model_path(path, base_path=None):
 
     raise ValueError("Could not identify model to load for '{}'".format(path))
 
-def prepare_domain_encoders(domain_encoder_paths, base_path):
-    """ Processes the domain encoder information from the benchmark specification. Handles
+def prepare_task_encoders(encoder_paths, base_path):
+    """ Processes the task encoder information from the benchmark specification. Handles
     relative paths or path placeholders (e.g., '%ccobra%' mapping to the module directory of
     the local CCOBRA installation).
 
     Parameters
     ----------
-    domain_encoder_paths : dict(str, str)
-        Dictionary mapping from domains to encoders.
+    encoder_paths : dict(str, str)
+        Dictionary mapping from domains to task encoders with absolute paths.
 
     Returns
     -------
     dict(str, str)
-        Dictionary mapping from domains to encoders with absolute paths.
+        Dictionary mapping from domains to paths containing task encoders.
 
     """
 
-    domain_encoders = {}
-    for domain, domain_encoder_path in domain_encoder_paths.items():
+    encs = {}
+    for domain, encoder_path in encoder_paths.items():
         # Normalize encoder path
-        domain_encoder_path = fix_rel_path(domain_encoder_path, base_path)
+        encoder_path = fix_rel_path(encoder_path, base_path)
 
         # To instantiate the encoder we need to change to its context (i.e., set the PATH variable
         # accordingly).
         enc = None
-        with contextmanager.dir_context(domain_encoder_path):
-            imp = modelimporter.ModelImporter(domain_encoder_path, superclass=CCobraDomainEncoder)
+        with contextmanager.dir_context(encoder_path):
+            imp = modelimporter.ModelImporter(encoder_path, superclass=encoders.CCobraTaskEncoder)
             enc = imp.instantiate()
 
         if not enc:
             raise ValueError('Failed to instantiate encoder class.')
-        domain_encoders[domain] = enc
+        encs[domain] = enc
 
-    return domain_encoders
+    return encs
+
+def prepare_resp_encoders(encoder_paths, base_path):
+    """ Processes the response encoder information from the benchmark specification. Handles
+    relative paths or path placeholders (e.g., '%ccobra%' mapping to the module directory of
+    the local CCOBRA installation).
+
+    Parameters
+    ----------
+    encoder_paths : dict(str, str)
+        Dictionary mapping from domains to response encoders with absolute paths.
+
+    Returns
+    -------
+    dict(str, str)
+        Dictionary mapping from domains to paths containing response encoders.
+
+    """
+
+    encs = {}
+    for domain, encoder_path in encoder_paths.items():
+        # Normalize encoder path
+        encoder_path = fix_rel_path(encoder_path, base_path)
+
+        # To instantiate the encoder we need to change to its context (i.e., set the PATH variable
+        # accordingly).
+        enc = None
+        with contextmanager.dir_context(encoder_path):
+            imp = modelimporter.ModelImporter(encoder_path, superclass=encoders.CCobraResponseEncoder)
+            enc = imp.instantiate()
+
+        if not enc:
+            raise ValueError('Failed to instantiate encoder class.')
+        encs[domain] = enc
+
+    return encs
+
+def prepare_comparator(comparator_path):
+    """ Processes the comparator path from the benchmark specification. Imports the object
+    dynamically.
+
+    Parameters
+    ----------
+    comparator_path : str
+        Path to the python script file containing the comparator definition.
+
+    Returns
+    -------
+    ccobra.benchmark.comparator.Comparator
+        Comparator object.
+
+    """
+
+    comp = None
+
+    with contextmanager.dir_context(comparator_path):
+        imp = modelimporter.ModelImporter(comparator_path, superclass=comparator.Comparator)
+        comp = imp.instantiate()
+
+    if not comp:
+        raise ValueError('Failed to instantiate comparator class.')
+
+    return comp
 
 class ModelInfo():
     """ Model information container. Contains the properties required to initialize and identify
@@ -176,9 +239,27 @@ class ModelInfo():
             self.load_specific_class = model_info.get('classname', self.load_specific_class)
 
     def __repr__(self):
+        """ Generates a string representation for the model info container.
+
+        Returns
+        -------
+        str
+            ModelInfo string representation.
+
+        """
+
         return str(self)
 
     def __str__(self):
+        """ Generates a string representation for the model info container.
+
+        Returns
+        -------
+        str
+            ModelInfo string representation.
+
+        """
+
         return 'path={}, override_name={}, load_specific_class={}, args={}'.format(
             self.path, self.override_name, self.load_specific_class, self.args)
 
@@ -231,10 +312,9 @@ class Benchmark():
 
         # Parse the JSON content
         self.parse_type()
-        self.parse_comparator()
-        self.parse_data()
+        self.parse_auxiliary_evaluations()
         self.parse_models()
-        self.parse_domain_encoders()
+        self.parse_data()
 
         # Verify
         if self.type == 'coverage':
@@ -252,21 +332,97 @@ class Benchmark():
             raise ValueError('Unsupported evaluation type: {}'.format(self.type))
         logger.debug('Evaluation type: %s', self.type)
 
-    def parse_comparator(self):
-        """ Parses the comparator information (equality, nvc).
+    def parse_comparator(self, comparator_str):
+        """ Parses the comparator information.
+
+        Parameters
+        ----------
+        comparator_str : str
+            Either is one of the library-defined comparator labels (equality, nvc, absdiff) or
+            a path to a comparator implementation to load dynamically.
+
+        Returns
+        -------
+        ccobra.benchmark.comparator.Comparator
+            Comparator object.
 
         """
 
-        # Extract comparator settings from benchmark description
-        comparator_type = self.json_content.get('comparator', 'equality')
-        logger.debug('Comparator type: %s', comparator_type)
-        if comparator_type == 'equality':
-            self.eval_comparator = comparator.EqualityComparator()
-        elif comparator_type == 'nvc':
-            self.eval_comparator = comparator.NVCComparator()
-        else:
-            raise ValueError('Invalid comparator type specified: {}'.format(comparator_type))
-        logger.debug('eval_comparator: %s', self.eval_comparator)
+        # Create the comparator instance
+        logger.debug('Comparator string: %s', comparator_str)
+        if comparator_str == 'equality':
+            return comparator.EqualityComparator()
+        elif comparator_str == 'nvc':
+            return comparator.NVCComparator()
+        elif comparator_str == 'absdiff':
+            return comparator.AbsDiffComparator()
+
+        logger.debug('Nonlabel comparator string: %s', comparator_str)
+
+        # Normalize path string
+        comparator_str = fix_rel_path(comparator_str, self.base_path)
+        logger.debug('Absolute comparator path: %s', comparator_str)
+
+        if not os.path.isfile(comparator_str):
+            raise ValueError('Comparator string is not a file: {}'.format(comparator_str))
+
+        return prepare_comparator(comparator_str)
+
+    def parse_auxiliary_evaluations(self):
+        """ Parses auxiliary evaluation configurations from the benchmark content.
+
+        """
+
+        evaluations = self.json_content.get('aux_evaluations', [])
+
+        task_encoders = self.json_content.get('task_encoders', {})
+        if 'syllogistic' not in task_encoders:
+            task_encoders['syllogistic'] = '%ccobra%/syllogistic/task_encoder_syl.py'
+        if 'propositional' not in task_encoders:
+            task_encoders['propositional'] = '%ccobra%/propositional/task_encoder_prop.py'
+
+        resp_encoders = self.json_content.get('response_encoders', {})
+        if 'syllogistic' not in resp_encoders:
+            resp_encoders['syllogistic'] = '%ccobra%/syllogistic/resp_encoder_syl.py'
+        if 'propositional' not in resp_encoders:
+            resp_encoders['propositional'] = '%ccobra%/propositional/resp_encoder_prop.py'
+
+        response_eval = {
+            'data_column': 'response',
+            'comparator': self.json_content.get('comparator', 'equality'),
+            'prediction_fn_name': 'predict',
+            'adapt_fn_name': 'adapt',
+            'task_encoders': task_encoders,
+            'response_encoders': resp_encoders
+        }
+        evaluations.insert(0, response_eval)
+
+        evaluation_handlers = []
+        evaluation_targets = []
+        for eva in evaluations:
+            task_encoders = None
+            if 'task_encoders' in eva:
+                task_encoders = prepare_task_encoders(eva['task_encoders'], self.base_path)
+
+            resp_encoders = None
+            if 'response_encoders' in eva:
+                resp_encoders = prepare_resp_encoders(eva['response_encoders'], self.base_path)
+
+            eh = evaluation_handler.EvaluationHandler(
+                data_column=eva['data_column'],
+                comparator=self.parse_comparator(eva.get('comparator', 'equality')),
+                predict_fn_name=eva['prediction_fn_name'],
+                adapt_fn_name=eva.get('adapt_fn_name', None),
+                task_encoders=task_encoders,
+                resp_encoders=resp_encoders
+            )
+            evaluation_handlers.append(eh)
+            evaluation_targets.append(eva['data_column'])
+
+            logger.debug('Added evaluation handler: %s', eh)
+
+        self.evaluation_handlers = evaluation_handlers
+        self.evaluation_targets = evaluation_targets
 
     def parse_data_path(self, path):
         """ Reads in a dataset CSV file and returns it as a pandas.DataFrame object. If a list
@@ -342,10 +498,10 @@ class Benchmark():
         logger.debug('corresponding_data: %s', self.corresponding_data)
 
         # Construct CCOBRA datasets
-        self.data_test = CCobraData(data_test_df)
-        self.data_pre_train = CCobraData(data_pre_train_df) if data_pre_train_df is not None else None
-        self.data_pre_train_person = CCobraData(data_pre_train_person_df) if data_pre_train_person_df is not None else None
-        self.data_pre_person_background = CCobraData(data_pre_person_background_df) if data_pre_person_background_df is not None else None
+        self.data_test = CCobraData(data_test_df, target_columns=self.evaluation_targets)
+        self.data_pre_train = CCobraData(data_pre_train_df, target_columns=self.evaluation_targets) if data_pre_train_df is not None else None
+        self.data_pre_train_person = CCobraData(data_pre_train_person_df, target_columns=self.evaluation_targets) if data_pre_train_person_df is not None else None
+        self.data_pre_person_background = CCobraData(data_pre_person_background_df, target_columns=self.evaluation_targets) if data_pre_person_background_df is not None else None
 
         # In case of non-corresponding datasets, make sure that identifiers do not overlap by
         # offsetting the training data (ensures that test identifiers remain identifiable)
@@ -366,7 +522,7 @@ class Benchmark():
                 domain_related_df = pd.concat(domain_related_df, data_pre_train_person_df)
 
             if not domain_related_df.empty:
-                self.data_pre_train_person = CCobraData(domain_related_df.drop(columns='_unique_id'))
+                self.data_pre_train_person = CCobraData(domain_related_df.drop(columns='_unique_id'), target_columns=self.evaluation_targets)
 
             # Append domain unrelated data to pre_person_background
             domain_unrelated_df = data_train_only_df.loc[~data_train_only_df['domain'].isin(self.data_test.domains)]
@@ -375,7 +531,7 @@ class Benchmark():
                 domain_unrelated_df = pd.concat(domain_unrelated_df, data_pre_person_background_df)
 
             if not domain_unrelated_df.empty:
-                self.data_pre_person_background = CCobraData(domain_unrelated_df.drop(columns='_unique_id'))
+                self.data_pre_person_background = CCobraData(domain_unrelated_df.drop(columns='_unique_id'), target_columns=self.evaluation_targets)
 
     def parse_models(self):
         """ Parses the benchmark model information.
@@ -386,24 +542,6 @@ class Benchmark():
         self.models = [ModelInfo(x, self.base_path) for x in self.json_content['models']]
         logger.debug('models:\n%s', '\n'.join([str(x) for x in self.models]))
 
-    def parse_domain_encoders(self):
-        """ Parses the benchmark domain encoder information.
-
-        """
-
-        # Parse domain encoder information
-        self.encoders = {}
-        if 'domain_encoders' in self.json_content:
-            self.encoders = prepare_domain_encoders(self.json_content['domain_encoders'], self.base_path)
-
-        # Include default encoders if not overridden
-        if 'syllogistic' not in self.encoders:
-            self.encoders['syllogistic'] = SyllogisticEncoder()
-        if 'propositional' not in self.encoders:
-            self.encoders['propositional'] = PropositionalEncoder()
-
-        logger.debug('Encoders:\n%s', self.encoders)
-
     def __str__(self):
         """ Generates a string representation of the benchmark information.
 
@@ -421,8 +559,8 @@ class Benchmark():
         s.append('   models:')
         for idx, model in enumerate(self.models):
             s.append('      ({}) {}'.format(idx + 1, model))
-        s.append('   encoders:')
-        for enc, val in self.encoders.items():
-            s.append('      {}: {}'.format(enc, val))
-        s.append('   comparator: {}'.format(self.eval_comparator))
+        s.append('   evaluation handlers:')
+        for eh in self.evaluation_handlers:
+            s.append('      {}'.format(str(eh)))
+        s.append('   evaluation targets: {}'.format(self.evaluation_targets))
         return '\n'.join(s)
