@@ -23,6 +23,8 @@ from . import benchmark as bmark
 from . import evaluator
 from .visualization import html_creator, viz_plot
 
+from ..version import __version__
+
 
 def parse_arguments():
     """ Parses the command line arguments for the benchmark runner.
@@ -34,7 +36,7 @@ def parse_arguments():
 
     """
 
-    parser = argparse.ArgumentParser(description='')
+    parser = argparse.ArgumentParser(description='CCOBRA version {}.'.format(__version__))
     parser.add_argument('benchmark', type=str, help='Benchmark file.')
     parser.add_argument('-m', '--model', type=str, help='Model file.')
     parser.add_argument(
@@ -121,14 +123,31 @@ def main(args):
     if 'save' in args:
         res_df.to_csv(args['save'], index=False)
 
-    # Run the metric visualizer
-    htmlcrtr = html_creator.HTMLCreator([
+    # Create metrics dictionary
+    #(TODO: check if there is a good way of dynamically specify the visualization)
+    default_list = [
         viz_plot.AccuracyVisualizer(),
         viz_plot.BoxplotVisualizer(),
-        viz_plot.MFATableVisualizer(),
-        viz_plot.SubjectTableVisualizer(),
-        viz_plot.ModelLogVisualizer()
-    ])
+        viz_plot.SubjectTableVisualizer()
+    ]
+    metrics = []
+    for idx, eva in enumerate(benchmark.evaluation_handlers):
+        if idx == 0:
+            metrics.append((
+                eva, [
+                    viz_plot.AccuracyVisualizer(),
+                    viz_plot.BoxplotVisualizer(),
+                    viz_plot.SubjectTableVisualizer(),
+                    viz_plot.MFATableVisualizer(),
+                    viz_plot.ModelLogVisualizer()
+                ]))
+        else:
+            metrics.append((
+                eva, default_list
+            ))
+
+    # Run the metric visualizer
+    htmlcrtr = html_creator.HTMLCreator(metrics)
 
     # Prepare the benchmark output information and visualize the evaluation results
     path_pre_train = ''
@@ -183,6 +202,12 @@ def entry_point():
 
     """
 
+    # Manually catch version command line argument
+    if len(sys.argv) > 1 and sys.argv[1] == '--version':
+        print('CCOBRA version {}'.format(__version__))
+        exit()
+
+    # Parse command line arguments
     args = parse_arguments()
 
     try:
